#include "test_glv.h"

using namespace glv;

const int pages = 8;
Buttons<1,pages> tabs(Rect(100, pages*20), true);
Group groups[pages];

Button bt1(Rect(20), true), bt2(bt1, true, draw::x, draw::minus);
Buttons<1,4> bts14(Rect(60, 60), true);
Buttons<4,1> bts41(Rect(120, 15), true);
Buttons<4,4> bts44(Rect(60));
Slider sl1H(Rect(100, 20)), sl1V(Rect(20, 100));
Slider2D sl2(Rect(100));
SliderGrid<3> sg3(Rect(100));
SliderGrid<4> sg4(Rect(100));


void drawCB(View * v){
	using namespace glv::draw;

	for(int i=0; i<tabs.size(); ++i) groups[i].property(Visible, tabs.value(i));

	push3D(v->w, v->h);		// push into 3-D mode passing in width and height of view
	
	pop3D();
}


int main (int argc, char ** argv){
	
	GLV top(drawCB);
	Window win(400, 300, "GLV Examples", &top);
	
	top.colors().back.set(1);
	
	glv::Style::standard().color.set(StyleColor::BlackOnWhite);
	
	for(int i=0; i<pages; ++i){
		//groups[i].visible = false;
		groups[i].disable(Visible);
		groups[i].stretch(1,1);
		top << groups[i];
	}
	
	groups[0] << bt1.pos(Place::BL).anchor(Place::CC)(Event::MouseDrag, Behavior::mouseResize);
	groups[0] << bt2.pos(Place::BR, -4).anchor(Place::CC)(Event::MouseDrag, Behavior::mouseResize);
	groups[1] << bts14.pos(Place::BR, -4).anchor(Place::CC);
	groups[1] << bts41.pos(Place::TL, 0, 4).anchor(Place::CC);
	groups[1] << bts44.pos(Place::BL).anchor(Place::CC);
	groups[2] << sl1H.pos(Place::BL).anchor(Place::CC);
	groups[2] << sl1V.pos(Place::BR, -4).anchor(Place::CC);
	groups[3] << sl2.pos(Place::BL).anchor(Place::CC);
	groups[4] << sg3.pos(Place::BL).anchor(Place::CC);
	groups[4] << sg4.pos(Place::TL, 0, 4).anchor(Place::CC);
	groups[5] << (new Label("Horizontal"))->pos(Place::BL).anchor(Place::CC);
	groups[5] << (new Label("Vertical", true))->pos(Place::BR,-16).anchor(Place::CC);

	tabs << (new Label("Button"		))->anchor(0.5, 1./(tabs.size()*2)).pos(Place::CC);
	tabs << (new Label("Buttons"	))->anchor(0.5, 3./(tabs.size()*2)).pos(Place::CC);
	tabs << (new Label("Slider"		))->anchor(0.5, 5./(tabs.size()*2)).pos(Place::CC);
	tabs << (new Label("Slider2D"	))->anchor(0.5, 7./(tabs.size()*2)).pos(Place::CC);
	tabs << (new Label("SliderGrid"	))->anchor(0.5, 9./(tabs.size()*2)).pos(Place::CC);
	tabs << (new Label("Label"	    ))->anchor(0.5,11./(tabs.size()*2)).pos(Place::CC);
	top << tabs;
	
	tabs(Event::MouseDrag, Behavior::mouseResize);
	
//	Color c;
//	c = HSV(0.5, 1, 1);
//	c.print();
//	HSV hsv(c);
//	printf("[%g %g %g]", hsv.h, hsv.s, hsv.v);

	#define PRINT_SIZE(x) printf("sizeof(%s) = %d\n", #x, sizeof(x));
	PRINT_SIZE(View) PRINT_SIZE(Notifier) PRINT_SIZE(Rect)

	Application::run();
}

